@@ -1,5 +1,5 @@
 from functools import lru_cache
-from typing import Dict, List, Iterable, Any, Union, Type
+from typing import Dict, List, Iterable, Any, Union
 
 import numpy as np
 import numpy.typing as npt
@@ -7,12 +7,12 @@ import tensorflow as tf  # type: ignore
 
 from code_loader.contract.datasetclasses import DatasetSample, DatasetBaseHandler, InputHandler, \
     GroundTruthHandler, PreprocessResponse, VisualizerHandler, VisualizerCallableReturnType, CustomLossHandler, \
-    PredictionTypeHandler, MetadataHandler, CustomLayerHandler
+    PredictionTypeHandler, MetadataHandler, CustomLayerHandler, MetricHandler
 from code_loader.contract.enums import DataStateEnum, TestingSectionEnum, DataStateType
 from code_loader.contract.exceptions import DatasetScriptException
 from code_loader.contract.responsedataclasses import DatasetIntegParseResult, DatasetTestResultPayload, \
     DatasetPreprocess, DatasetSetup, DatasetInputInstance, DatasetOutputInstance, DatasetMetadataInstance, \
-    VisualizerInstance, PredictionTypeInstance, ModelSetup, CustomLayerInstance
+    VisualizerInstance, PredictionTypeInstance, ModelSetup, CustomLayerInstance, MetricInstance
 from code_loader.leap_binder import global_leap_binder
 from code_loader.utils import get_root_exception_line_number, get_shape
 
@@ -27,7 +27,16 @@ class LeapLoader:
             global_variables: Dict[Any, Any] = {}
             exec(self.dataset_script, global_variables)
         except Exception as e:
-            raise DatasetScriptException from e
+            raise DatasetScriptException(getattr(e, 'message', repr(e))) from e
+
+    @lru_cache()
+    def metric_by_name(self) -> Dict[str, MetricHandler]:
+        self.exec_script()
+        setup = global_leap_binder.setup_container
+        return {
+            metric_handler.name: metric_handler
+            for metric_handler in setup.metrics
+        }
 
     @lru_cache()
     def visualizer_by_name(self) -> Dict[str, VisualizerHandler]:
@@ -82,6 +91,10 @@ class LeapLoader:
             test_payloads.extend(handlers_test_payloads)
             is_valid = all([payload.is_passed for payload in test_payloads])
             setup_response = self.get_dataset_setup_response()
+        except DatasetScriptException as e:
+            line_number = get_root_exception_line_number()
+            general_error = f"Something went wrong, {repr(e.__cause__)} line number: {line_number}"
+            is_valid = False
         except Exception as e:
             line_number = get_root_exception_line_number()
             general_error = f"Something went wrong, {repr(e)} line number: {line_number}"
@@ -198,23 +211,25 @@ class LeapLoader:
         metadata = [DatasetMetadataInstance(name=metadata.name, type=metadata.type)
                     for metadata in setup.metadata]
 
-        visualizers = [VisualizerInstance(visualizer_handler.name, visualizer_handler.type, visualizer_handler.arg_names)
-                    for visualizer_handler in setup.visualizers]
+        visualizers = [
+            VisualizerInstance(visualizer_handler.name, visualizer_handler.type, visualizer_handler.arg_names)
+            for visualizer_handler in setup.visualizers]
 
         custom_loss_names = [custom_loss.name for custom_loss in setup.custom_loss_handlers]
 
         prediction_types = []
         for prediction_type in setup.prediction_types:
-            custom_metrics_names = None
-            if prediction_type.custom_metrics:
-                custom_metrics_names = [custom_metric.__name__ for custom_metric in prediction_type.custom_metrics]
-            pred_type_inst = PredictionTypeInstance(prediction_type.name, prediction_type.labels,
-                                                    prediction_type.metrics, custom_metrics_names)
+            pred_type_inst = PredictionTypeInstance(prediction_type.name, prediction_type.labels)
             prediction_types.append(pred_type_inst)
 
+        metrics = []
+        for metric in setup.metrics:
+            metric_inst = MetricInstance(metric.name, metric.arg_names)
+            metrics.append(metric_inst)
+
         return DatasetSetup(preprocess=dataset_preprocess, inputs=inputs, outputs=ground_truths, metadata=metadata,
                             visualizers=visualizers, prediction_types=prediction_types,
-                            custom_loss_names=custom_loss_names)
+                            custom_loss_names=custom_loss_names, metrics=metrics)
 
     @staticmethod
     def get_model_setup_response() -> ModelSetup:
